# frozen_string_literal: true

require_relative "datetimecomponent/version"
require 'view_component/engine'
require 'action_view'
require 'action_pack'

module Datetime
  class Error < StandardError; end
  class DatetimeComponent < ViewComponent::Base
    FULL_DATETIME_ARGS = { null: '', short: '%a, %-d %b %y, %H:%M %Z', long: '%A, %-d %B %Y, %-l:%M %P %Z' }.freeze
    DAY_ARGS = { null: '', numeric: '%-d' }.freeze
    WEEKDAY_ARGS = { null: '', short: '%a', long: '%A'}.freeze
    MONTH_ARGS = { null: '', short: '%b', long: '%B', numeric: '%-m' }.freeze
    YEAR_ARGS = { null: '', short: '%-y', long: '%-Y' }.freeze
    DISPLAY_TIME_ARGS = { null: '', h12: '%-l:%M', h12meridian: '%-l:%M %P', h24: '%H:%M' }.freeze
  
    def initialize(custom_datetime_format: '', full_datetime: :null, date_separator: '/', weekday: :null,
                   day: :null, month: :null, year: :null, display_time: :null, display_timezone: false,
                   display_zone_offset: false, time_zone: 'UTC')
      super
      @_routes = nil
      @date_separator = date_separator
      @time_zone = TZInfo::Timezone.get(time_zone)
      @datetime_format = custom_datetime_format
      return unless @datetime_format.empty?
  
      @format_args = {}
      @format_args[:full_datetime] = FULL_DATETIME_ARGS[full_datetime]
      @format_args[:weekday] = WEEKDAY_ARGS[weekday]
      @format_args[:day] = DAY_ARGS[day]
      @format_args[:month] = MONTH_ARGS[month]
      @format_args[:year] = YEAR_ARGS[year]
      @format_args[:display_time] = DISPLAY_TIME_ARGS[display_time]
      @format_args[:display_timezone] = display_timezone ? '%Z' : ''
      @format_args[:zone_offset] = display_zone_offset ? 'UTC %:z' : ''
      @datetime_format = build_datetime_format
    end
  
    private
  
    # rubocop:enable Metrics/ParameterLists
    def build_datetime_format
      datetime_format = @format_args[:full_datetime]
      return "#{datetime_format} #{@format_args[:zone_offset]}" unless datetime_format.empty?
  
      datetime_format += @format_args[:weekday]
      datetime_format += ', ' if !datetime_format.empty? && !@format_args[:day].empty?
  
      datetime_format += build_dateformat
      datetime_format += ', ' if !datetime_format.empty? && !@format_args[:display_time].empty?
  
      datetime_format += build_timeformat
      datetime_format
    end
  
    def build_dateformat
      date_format = @format_args[:day]
  
      unless @format_args[:month].empty?
        date_format += @format_args[:month] == MONTH_ARGS[:numeric] ? @date_separator : ' '
        date_format += @format_args[:month]
      end
  
      unless @format_args[:year].empty?
        date_format += @format_args[:month] == MONTH_ARGS[:numeric] ? @date_separator : ' '
        date_format += @format_args[:year]
      end
      date_format
    end
  
    def build_timeformat
      time_format = @format_args[:display_time]
      time_format += ' ' unless @format_args[:display_time].empty?
      time_format += @format_args[:display_timezone]
      time_format += ' ' unless @format_args[:display_timezone].empty?
      time_format += @format_args[:zone_offset]
      time_format
    end
  end
end